package com.tzld.longarticle.recommend.server.service;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.*;
import com.tzld.longarticle.recommend.server.service.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.rank.RankService;
import com.tzld.longarticle.recommend.server.service.recall.RecallParam;
import com.tzld.longarticle.recommend.server.service.recall.RecallResult;
import com.tzld.longarticle.recommend.server.service.recall.RecallService;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecommendService {

    @Autowired
    private RecallService recallService;
    @Autowired
    private RankService rankService;

    @ApolloJsonValue("${accountStrategyConfig:{}}")
    private Map<String, String> accountStrategyConfigMap;


    public RecommendResponse recommend(RecommendRequest request) {

        RecommendParam param = genRecommendParam(request);
        log.info("genRecommendParam {}", JSONUtils.toJson(param));
        // 获取账号排序设置
        String strategyConfig = accountStrategyConfigMap.get(request.getAccountName());
        if (StringUtils.hasText(strategyConfig)) {
            param.setStrategy(strategyConfig);
        }
        RecallResult recallResult = recallService.recall(convertToRecallParam(param));
        log.info("recallResult {}", JSONUtils.toJson(recallResult));

        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        log.info("rankResult {}", JSONUtils.toJson(rankResult));

        return buildRecommendResponse(recallResult, rankResult, param.getPublishNum());
    }

    private RecommendResponse buildRecommendResponse(RecallResult recallResult, RankResult rankResult, Integer publishNum) {
        List<Content> contentList = rankResult.getContents();
        List<ArticleSortResponseDataItem> rankList = new ArrayList<>();
        List<ArticleSortResponseDataItem> filterContentList = new ArrayList<>();
        for (RecallResult.RecallData data : recallResult.getData()) {
            if (CollectionUtils.isNotEmpty(data.getFilterContents())) {
                for (Content filterContent : data.getFilterContents()) {
                    ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                    BeanUtils.copyProperties(filterContent, item);
                    filterContentList.add(item);
                }
            }
        }
        ArticleSortResponseData data = new ArticleSortResponseData();
        if (CollectionUtils.isNotEmpty(contentList)) {
            for (Content content : contentList) {
                ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                BeanUtils.copyProperties(content, item);
                rankList.add(item);
            }
        }
        data.setRank_list(rankList.subList(0, Math.min(rankList.size(), publishNum)));
        data.setFilter_list(filterContentList);

        RecommendResponse response = new RecommendResponse();
        response.setCode(0);
        response.setMsg("success");
        response.setData(data);
        return response;
    }

    public RecommendParam genRecommendParam(RecommendRequest request) {
        RecommendParam param = new RecommendParam();
        BeanUtils.copyProperties(request, param);
        return param;
    }

    public RecallParam convertToRecallParam(RecommendParam param) {
        RecallParam recallParam = new RecallParam();
        BeanUtils.copyProperties(param, recallParam);
        recallParam.setStrategy(param.getStrategy());
        recallParam.setGhId(param.getGhId());
        return recallParam;
    }


    public RankParam convertToRankParam(RecommendParam param, RecallResult recallResult) {
        RankParam rankParam = new RankParam();
        List<Content> contentList = new ArrayList<>();
        List<Content> backup = new ArrayList<>();
        if (Objects.nonNull(recallResult) && CollectionUtils.isNotEmpty(recallResult.getData())) {
            for (RecallResult.RecallData item : recallResult.getData()) {
                if (item.isBackup()) {
                    backup.addAll(item.getContents());
                } else {
                    contentList.addAll(item.getContents());
                }
            }

        }
        rankParam.setContents(contentList);
        rankParam.setBackup(backup);
        rankParam.setStrategy(param.getStrategy());
        rankParam.setAccountName(param.getAccountName());
        rankParam.setSize(param.getPublishNum());
        return rankParam;
    }

}
