package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.google.common.reflect.TypeToken;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.ArticlePreDistributeAccount;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticlePreDistributeAccountRepository;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @author dyp
 */
@Component
@Slf4j
public class AccountPreDistributeStrategy implements FilterStrategy {
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private ArticlePreDistributeAccountRepository articlePreDistributeAccountRepository;

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> contents = param.getContents();
        List<Content> filterContents = new ArrayList<>();
        Set<String> articles = new HashSet<>();
        String[] pools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        if (pools.length >= 3) {
            try {
                ArticlePreDistributeAccount optional = articlePreDistributeAccountRepository.findFirstByGhIdOrderByDateDesc(param.getGhId());
                if (optional != null && StringUtils.isNotEmpty(optional.getArticleList())) {
                    List<String[]> list = JSONUtils.fromJson(optional.getArticleList(), new TypeToken<List<String[]>>() {
                    }, Collections.emptyList());
                    for (String[] s : list) {
                        articles.add(s[0]);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        for (Content content : contents) {
            if (StringUtils.equals(pools[2], content.getContentPoolType())) {
                if (articles.contains(content.getCrawlerChannelContentId())) {
                    result.add(content.getId());
                }
            } else {
                result.add(content.getId());
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        log.info("AccountPreDistributeStrategy cost:{}", System.currentTimeMillis() - start);
        return filterResult;
    }

}
